package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.SignerWithRecovery;
import cn.com.infosec.crypto.digests.RIPEMD128Digest;
import cn.com.infosec.crypto.digests.RIPEMD160Digest;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.params.RSAKeyParameters;

public class ISO9796d2Signer implements SignerWithRecovery {
   public static final int TRAILER_IMPLICIT = 188;
   public static final int TRAILER_RIPEMD160 = 12748;
   public static final int TRAILER_RIPEMD128 = 13004;
   public static final int TRAILER_SHA1 = 13260;
   private Digest digest;
   private AsymmetricBlockCipher cipher;
   private int trailer;
   private int keyBits;
   private byte[] block;
   private byte[] mBuf;
   private int messageLength;
   private boolean fullMessage;
   private byte[] recoveredMessage;

   public ISO9796d2Signer(AsymmetricBlockCipher cipher, Digest digest, boolean implicit) {
      this.cipher = cipher;
      this.digest = digest;
      if (implicit) {
         this.trailer = 188;
      } else if (digest instanceof SHA1Digest) {
         this.trailer = 13260;
      } else if (digest instanceof RIPEMD160Digest) {
         this.trailer = 12748;
      } else {
         if (!(digest instanceof RIPEMD128Digest)) {
            throw new IllegalArgumentException("no valid trailer for digest");
         }

         this.trailer = 13004;
      }

   }

   public ISO9796d2Signer(AsymmetricBlockCipher cipher, Digest digest) {
      this(cipher, digest, false);
   }

   private boolean isSameAs(byte[] a, byte[] b) {
      int var3;
      byte[] var4;
      if ((var3 = this.messageLength) > (var4 = this.mBuf).length) {
         if (var4.length > b.length) {
            return false;
         }

         for(var3 = 0; var3 != this.mBuf.length; ++var3) {
            if (a[var3] != b[var3]) {
               return false;
            }
         }
      } else {
         if (var3 != b.length) {
            return false;
         }

         for(int var5 = 0; var5 != b.length; ++var5) {
            if (a[var5] != b[var5]) {
               return false;
            }
         }
      }

      return true;
   }

   private void clearBlock(byte[] block) {
      for(int var2 = 0; var2 != block.length; ++var2) {
         block[var2] = 0;
      }

   }

   public void init(boolean forSigning, CipherParameters param) {
      RSAKeyParameters param1;
      RSAKeyParameters var10002 = param1 = (RSAKeyParameters)param;
      this.cipher.init(forSigning, param1);
      byte[] forSigning1;
      this.block = forSigning1 = new byte[((this.keyBits = var10002.getModulus().bitLength()) + 7) / 8];
      if (this.trailer == 188) {
         this.mBuf = new byte[forSigning1.length - this.digest.getDigestSize() - 2];
      } else {
         this.mBuf = new byte[forSigning1.length - this.digest.getDigestSize() - 3];
      }

      this.reset();
   }

   public void update(byte b) {
      this.digest.update(b);
      int var2;
      byte[] var3;
      if ((var2 = this.messageLength) < (var3 = this.mBuf).length) {
         var3[var2] = b;
      }

      this.messageLength = var2 + 1;
   }

   public void update(byte[] in, int off, int len) {
      this.digest.update(in, off, len);
      int var5;
      byte[] var6;
      if (this.messageLength < this.mBuf.length) {
         for(int var4 = 0; var4 < len && var4 + (var5 = this.messageLength) < (var6 = this.mBuf).length; ++var4) {
            var5 += var4;
            var6[var5] = in[off + var4];
         }
      }

      this.messageLength += len;
   }

   public void reset() {
      this.digest.reset();
      this.messageLength = 0;
      this.clearBlock(this.mBuf);
      byte[] var1;
      if ((var1 = this.recoveredMessage) != null) {
         this.clearBlock(var1);
      }

      this.recoveredMessage = null;
      this.fullMessage = false;
   }

   public byte[] generateSignature() {
      int var1 = this.digest.getDigestSize();
      int var2;
      byte[] var3;
      int var4;
      int var8;
      byte[] var10000;
      if (this.trailer == 188) {
         var2 = 8;
         var4 = (var3 = this.block).length - var1 - 1;
         this.digest.doFinal(var3, var4);
         var10000 = this.block;
         var10000[var10000.length - 1] = -68;
      } else {
         var2 = 16;
         var4 = (var3 = this.block).length - var1 - 2;
         this.digest.doFinal(var3, var4);
         var10000 = var3 = this.block;
         int var5 = var3.length - 2;
         int var6;
         int var10001 = var6 = this.trailer;
         var3[var5] = (byte)(var6 >>> 8);
         var8 = var3.length - 1;
         var10000[var8] = (byte)var10001;
      }

      byte var7;
      if ((var2 = (var1 + (var1 = this.messageLength)) * 8 + var2 + 4 - this.keyBits) > 0) {
         int var10002 = var1 -= (var2 + 7) / 8;
         var7 = 96;
         var8 = var4 - var10002;
         System.arraycopy(this.mBuf, 0, this.block, var8, var1);
      } else {
         var7 = 64;
         var8 = var4 - var1;
         System.arraycopy(this.mBuf, 0, this.block, var8, var1);
      }

      if ((var1 = var8 - 1) > 0) {
         for(var8 = var1; var8 != 0; --var8) {
            this.block[var8] = -69;
         }

         var10000 = this.block;
         var10000[var1] = (byte)(var10000[var1] ^ 1);
         var10000[0] = 11;
         var10000[0] |= var7;
      } else {
         var10000 = this.block;
         var10000[0] = 10;
         var10000[0] |= var7;
      }

      byte[] var9 = this.block;
      var10000 = this.cipher.processBlock(var9, 0, var9.length);
      this.clearBlock(this.mBuf);
      this.clearBlock(this.block);
      return var10000;
   }

   public boolean verifySignature(byte[] param1) {
      // $FF: Couldn't be decompiled
   }

   public boolean hasFullMessage() {
      return this.fullMessage;
   }

   public byte[] getRecoveredMessage() {
      return this.recoveredMessage;
   }
}
